#MIT License
#Copyright (c) 2018 phR0ze
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in all
#copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#SOFTWARE.

class Module

  # Leveraged mattr_reader, mattr_writer, mattr_accessor from rails project on github
  # MIT license https://opensource.org/licenses/MIT
  # https://github.com/rails/rails/blob/2-1-stable/activesupport/lib/active_support/core_ext/module/attribute_accessors.rb
  #
  # Extends the module object with module and instance accessors for class attributes, 
  # just like the native attr* accessors for instance attributes.
  #
  # module AppConfiguration
  #   mattr_accessor :google_api_key
  #   self.google_api_key = "123456789"
  # end
  # AppConfiguration.google_api_key = "overriding the api key!"
  
  # Create reader class/module methods for property
  # @param syms [Array(Symbol)] splat array of property names
  def mattr_reader(*syms)
    syms.each{|sym|
      next if sym.is_a?(Hash)
      class_eval(<<-EOS, __FILE__, __LINE__)
        @@#{sym} = nil if !defined?(@@#{sym})
        
        def self.#{sym}
          @@#{sym}
        end
        def #{sym}
          @@#{sym}
        end
      EOS
    }
  end
  
  # Create reader class/module methods for property
  # @param syms [Array(Symbol)] splat array of property names
  def mattr_writer(*syms)
    syms.each{|sym|
      class_eval(<<-EOS, __FILE__, __LINE__)
        @@#{sym} = nil if !defined?(@@#{sym})
        
        def self.#{sym}=(obj)
          @@#{sym} = obj
        end
        def #{sym}=(obj)
          @@#{sym} = obj
        end
      EOS
    }
  end
  
  def mattr_accessor(*syms)
    mattr_reader(*syms)
    mattr_writer(*syms)
  end
end
